##########################################################################
# Regression Analysis (I)                                                #
# Kutner et al. (2019), Applied Linear Statistical Models:               #
#  Applied Linear Regression Models, Mcgraw-Hill Inc., (5th edition)     #
# Chapter 5 ~ 14                                                         #
# d~ ߬FvjǲέpǨt                                            #
# http://www.hmwu.idv.tw/                                                #
##########################################################################

# ALSM: https://cran.r-project.org/web/packages/ALSM/index.html 


##########################################################################
# Chapter 5: Matrix Approach to Simple Linear Regression Analysis        #
##########################################################################


#
# 5.1 Matrices
#

A <- matrix(c(2, 7, 3, 5, 10, 4), ncol = 2)
A
dim(A)
nrow(A)
ncol(A)
str(A)
class(A)
A[2, 1]
A[2, ]
A[, 1]


# Transpose
t(A)


# Equality of Matrices
B <- A
B == A
sum((A - B)^2)



#
# 5.2 Matrix Addition and Subtraction
# 
set.seed(12345)
M1 <- matrix(sample(0:100, 20), ncol = 4)
M1
M2 <- matrix(sample(0:100, 20), ncol = 4)  
M2

dim(M1)
dim(M2)

M2 - M1
M2 + M1



#
# 5.3 Matrix Multiplication
#

# Multiplication of a Matrix by a Scalar
2 * M1

# M1 %*% M2 # Error
# M2 %*% M1 # Error

M1 %*% t(M2)
M2 %*% t(M1)


Y <- matrix(sample(0:100, 5), ncol = 1)
dim(Y)
M <- t(Y) %*% Y
M
dim(M)
str(M)
class(M)
M[1, 1]



#
# 5.4 Special Types of Matrices
#

# Symmetric Matrix
S <- cov(iris[, 1:4])
S
t(S)


# Diagonal Matrix
diag(S)


# Identity Matrix
I <- diag(5)
I


# Vector and Matrix with All Elements Unity
n <- 5
Ones.vector <- matrix(1, nrow = n, ncol = 1)  # 1
Ones.vector
Ones.matrix <- matrix(1, nrow = n, ncol = n)  # J
Ones.matrix

t(Ones.vector) %*% Ones.vector  
Ones.vector %*% t(Ones.vector)  # J


# Zero Vector
Zeros.Vector <- matrix(0, nrow = n, ncol = 1)
Zeros.Vector


#
# 5.5 Linear Dependence and Rank of Matrix
#

# detect.lindep {plm}
# install.packages("plm")
library(plm)
A <- matrix(c(1, 2, 3, 2, 2, 4, 5, 10, 15, 1, 6, 1), nrow = 3)
A
detect.lindep(A)


# Rank of Matrix
# install.packages("Matrix")
library(Matrix)
rankMatrix(A)
rankMatrix(A)[1]

qr(A)$rank



#
# 5.6 Inverse of a Matrix
#

a1 <- c(3, 2, 5)
a2 <- c(2, 3, 2)
a3 <- c(5, 2, 4)
A <- rbind(a1, a2, a3)
print(A)

det(A) # if determinnat of A != 0, inverse exists
rankMatrix(A)[1] # full rank, inverse exists

A.inverse <- solve(A)
A.inverse %*% A
A %*% A.inverse 






##########################################################################
# Chapter 6: Multiple Regression (I)                                     #
##########################################################################

# Example
# Dwaine Studios, Inc., operates portrait studios in 21 cities (n = 21)
# of medium size. These studios specialize in portraits of children. The company is
# considering an expansion into other cities of medium size and wishes to investigate
# whether sales (Y or SALES, in thousands of dollars) in a community can be predicted
# from the number of persons aged 16 or younger in the community (X1 or
# TARGTPOP for target population) and the per capita disposable 
# personal income in the community (X2 or DISPOINC for disposable income
# in thousands of dollars).

# Y or SALES: in thousands of dollars in a community can be predicted
# X1 or TARGTPOP: the number of persons aged 16 or younger in the community (target population)
# X2 or DISPOINC: for disposable income in thousands of dollars).

DSI <- read.csv("data/CH06FI05.csv", header = F)
colnames(DSI) <- c("TARGTPOP", "DISPOINC", "SALES")
DSI <- DSI[, c(3, 1, 2)]


# Figure 6.4: Scatter plot matrix
pairs(DSI)


# correlation matrix
cor(DSI)


# The first-order regression model
DSI.lm <- lm(SALES ~ TARGTPOP + DISPOINC, data = DSI)
DSI.lm


# Regression output
summary(DSI.lm)


# ANOVA table
anova(DSI.lm)


# Figure 6.6: spining plot
# install.packages("rgl")
library(rgl)
attach(DSI)
plot3d(TARGTPOP, DISPOINC, SALES)
plot3d(TARGTPOP, DISPOINC, SALES, type = "s", col = "red", size = 2)


# Basic Calculations

DSI
X <- as.matrix(cbind(1, DSI[, c(2, 3)]))
X
Y <- matrix(DSI[, 1], ncol = 1)


# (XX)^{???1}
solve(t(X) %*% X)


# XY
t(X) %*% Y


# The least squares estimates b
B <-   solve(t(X) %*% X) %*% (t(X) %*% Y)  
B


# Figure 6.7: A 3D plot of the estimated regression function

plot3d(TARGTPOP, DISPOINC, SALES, type = "s", col = "red", size = 2)
coefs <- coef(DSI.lm)
a <- coefs["TARGTPOP"]
b <- coefs["DISPOINC"]
c <- -1
d <- coefs["(Intercept)"]

# planes3d draw planes using the parametrization a x + b y + c z + d = 0
planes3d(a, b, c, d, alpha = 0.5)


# The fitted values
fitted(DSI.lm)


# The residuals
residuals(DSI.lm)

# same as 
names(DSI.lm)
DSI.lm$fitted.values
DSI.lm$residuals


# Figure 6.8: residual plots
par(mfrow = c(2, 2))
plot(DSI.lm$fitted.values, DSI.lm$residuals, pch = 16,
     xlab = "Fitted", ylab = "Residual",
     main = "Residual Plot againist Y.hat")
plot(DSI$TARGTPOP, DSI.lm$residuals, pch = 16,
     xlab = "TARGTPOP", ylab = "Residual",
     main = "Residual Plot againist X1")
plot(DSI$DISPOINC, DSI.lm$residuals, pch = 16,
     xlab = "DISPOINC", ylab = "Residual",
     main = "Residual Plot againist X2")
plot(DSI$TARGTPOP * DSI$DISPOINC, DSI.lm$residuals, pch = 16,
     xlab = "X1X2", ylab = "Residual",
     main = "Residual Plot againist X1X2")



# Figure 6.9: A plot of the absolute residuals against the ???tted values
par(mfrow = c(1, 2))
plot(DSI.lm$fitted.values, DSI.lm$residuals, pch = 16,
     xlab = "Fitted", ylab = "Abs(Residuals)",
     main = "Plot of Absolute Residuals againist Y.hat")
qqnorm(DSI.lm$residuals, pch = 16,
     xlab = "Expected", ylab = "Residual",
     main = "Normal Probability Plot")


# The correlation between the ordered residuals and their expected values
DSI.qqplot <- qqnorm(DSI.lm$residuals, pch = 16,
                     xlab = "Expected", ylab = "Residual",
                     main = "Normal Probability Plot", plot.it = F) 

cor(DSI.qqplot$x, DSI.qqplot$y)



# diagnostic plots
plot(DSI.lm)



# Analysis of Variance
# Regression output
DSI.lm.output <- summary(DSI.lm)
DSI.lm.output
str(DSI.lm.output)
DSI.lm.output$r.squared



# ANOVA table
DSI.lm.anova <- anova(DSI.lm) # Sequential (type I) SS
DSI.lm.anova
str(DSI.lm.anova)
names(DSI.lm.anova)
DSI.lm.anova$"F value"
DSI.lm.anova$"Pr(>F)" 

# saem as
DSI.aov <- aov(SALES ~ TARGTPOP + DISPOINC, data = DSI)
DSI.aov
summary(DSI.aov)







##########################################################################
# Chapter 7: Multiple Regression (II)                                    #
##########################################################################


#
# 7.1 Extra Sums of Squares
#

#(Table 7.1)
# A sample of 20 healthy females 25 ??? 34 years old. 
# Y: body fat 
# X1: triceps skinfold thickness (TY٥֤Uתիp) 
# X2: thigh circumference (jL)
# X3: midarm circumference (u)


BF.data <- read.csv("data/CH07TA01.csv", header = F)
BF.data
colnames(BF.data) <- c("X1", "X2", "X3", "BodyFat")
BF.data

m1 <- lm(BodyFat ~ X1, data = BF.data)
m2 <- lm(BodyFat ~ X2, data = BF.data)
m12 <- lm(BodyFat ~ X1 + X2, data = BF.data)
m123 <- lm(BodyFat ~ X1 + X2 + X3, data = BF.data)

BF.model <- list(m1, m2, m12, m123)
lapply(BF.model, summary)
lapply(BF.model, anova)


#(Table 7.2)(c)(d)
m0 <- lm(BodyFat ~ 1, data = BF.data)
anova(m0, m12)
anova(m0, m123)




#(Table 7.4)
# get extra sum of squares
m123.anova <- anova(m123)
m123.anova

#Analysis of Variance Table
#
#Response: BodyFat
#Df Sum Sq Mean Sq F value    Pr(>F)    
# X1         1 352.27  352.27 57.2768 1.131e-06 ***
# X2         1  33.17   33.17  5.3931   0.03373 *  
# X3         1  11.55   11.55  1.8773   0.18956    
#Residuals 16  98.40    6.15                      
#---
#  Signif. codes:  0 *** 0.001 ** 0.01 * 0.05 . 0.1   1

str(m123.anova)
m123.anova$`Sum Sq`   # SSR(X1), SSR(X2|X1), SSR(X3|X1, X2), SSE(X1, X2, X3)



#
# 7.2 Uses of Extra Sums of Squares in Tests for Regression Coefficients
#

# Test whether a Single beta_k = 0
m123.anova
m123.anova$`F value`  # test H0: beta1 = 0; H0: beta2 = 0; H0: beta3 = 0


# Test whether Several ]k = 0
# H0 : beta2 = beta3 = 0
# m1: reduced model
# m123: full model
anova(m1, m123)




#
# 7.4 Coefficients of Partial Determination
#


# coefficient of partial determination between Y and X1, 
# given that X2 is in the mode1
# Body Fat Example
anova.full <- anova(m12)
anova.reduced <- anova(m1)
sse.full <- tail(anova.full$"Sum Sq", 1) # SSE(X1, X2)
sse.reduced <- tail(anova.reduced$"Sum Sq", 1) # SSE(X1)
R2_YX2_given_X1 <- (sse.reduced - sse.full) / sse.reduced
R2_YX2_given_X1


anova.full <- anova(m123)
anova.reduced <- anova(m12)
sse.full <- tail(anova.full$"Sum Sq", 1) # SSE(X1, X2)
sse.reduced <- tail(anova.reduced$"Sum Sq", 1) # SSE(X1)
R2_YX3_given_X1X2 <- (sse.reduced - sse.full) / sse.reduced
R2_YX3_given_X1X2


#install.packages("asbio")
library(asbio)
partial.R2(m12, m123) # partial.R2(reduced_model, full_model)



#
# 7.6 Multicollinearity and Its effects
#


# skip, since the same skill has been used in the previous sections
# scatterplot matrix: paris
# extra sum of squares






############################################################################
# Chapter 8: Regression Models for Quantitative and Qualitative Predictors #
############################################################################



# Setting. A researcher studied the effects of the charge rate and temperature on
# the life of a new type of power cell in a preliminary small-scale expetiment. 
# X1: charge rate, was controlled at three levels (0.6, 1.0, and 1.4 amperes (w))
# X2: ambient temperature, was controlled at three levels (l0, 20, 30???C).
# Y: life of the power cell, the number of discharge - charge cycles


PowerCell.data <- read.csv("data/CH08TA01.csv", header = F)
PowerCell.data
colnames(PowerCell.data) <- c("Cycles", "ChargeRate", "Temperature")
PowerCell.data
PowerCell.data$x1 <- (PowerCell.data$ChargeRate - mean(PowerCell.data$ChargeRate))/0.4
PowerCell.data$x2 <- (PowerCell.data$Temperature - mean(PowerCell.data$Temperature))/10

PowerCell.data

cor(PowerCell.data$ChargeRate, PowerCell.data$ChargeRate^2)
cor(PowerCell.data$x1, PowerCell.data$x1^2)

cor(PowerCell.data$Temperature, PowerCell.data$Temperature^2)
cor(PowerCell.data$x2, PowerCell.data$x2^2)


# Figure 8.4
# fit the second-order polynomial regression model
PowerCell.lm <- lm(Cycles ~ x1 * x2 + I(x1^2) + I(x2^2), data = PowerCell.data)
PowerCell.lm
summary(PowerCell.lm)
anova(PowerCell.lm)

PowerCell.lm0 <- lm(Cycles ~ 1, data = PowerCell.data)
anova(PowerCell.lm0, PowerCell.lm)



# Figure 8.5: residual plot
par(mfrow = c(2, 2))
plot(PowerCell.lm$fitted.values, PowerCell.lm$residuals, pch = 16,
     xlab = "Fitted", ylab = "Residual",
     main = "Residual Plot againist Y.hat")
plot(PowerCell.data$x1, PowerCell.lm$residuals, pch = 16,
     xlab = "x1", ylab = "Residual",
     main = "Residual Plot againist x1")
plot(PowerCell.data$x2, PowerCell.lm$residuals, pch = 16,
     xlab = "x2", ylab = "Residual",
     main = "Residual Plot againist x2")
qqnorm(PowerCell.lm$residuals, xlab = "Expected", ylab = "Residual",
       main = "Normal probability Plot", pch = 16)




# lack of fit test
full.model <- lm(Cycles ~ x1 * x2 + I(x1^2) + I(x2^2), data = PowerCell.data)
reduced.model <- lm(Cycles ~ factor(x1) * factor(x2), data = PowerCell.data)
anova(full.model, reduced.model)
# SSLF = SSE ??? SSPE 
#      = 5240.44 ??? 1404.67 = 3835.77


# Coefficient of Multiple Determination
full.model.s <- summary(full.model)
full.model.s
str(full.model.s)
full.model.s$r.squared
full.model.s$adj.r.squared




# Partial F Test
# H0 : beta_11 = beta_22 = beta_12 = 0
reduced.model <- lm(Cycles ~ x1 + x2, data = PowerCell.data)
anova(full.model, reduced.model)
# F??? = 0.78



# final model: First-Order Model
PowerCell.folm <- lm(Cycles ~ x1 + x2, data = PowerCell.data)
PowerCell.folm
summary(PowerCell.folm)
anova(PowerCell.folm)



#
# 8.3 Qualitative Predictors
#

# Example In a study of innovation in the insurance industry, an economist wished
# to relate the speed with which a particular insurance innovation is adopted (Y ) to
# X1: the size of the insurance firm 
# X2: the type of firm
#  Y: the number of months elapsed between the time the first firm adopted the
#     innovation and the time the given firm adopted the innovation



# Table 8.2
insurance.data <- read.csv("data/CH08TA02.csv", header = F)
colnames(insurance.data) <- c("Y", "X1", "X2")
str(insurance.data)
insurance.data$X2 <- as.factor(insurance.data$X2)
str(insurance.data)


# Table 8.3
insurance.lm <- lm(Y ~ X1 + X2, data = insurance.data)
summary(insurance.lm)
anova(insurance.lm)




#
# 8.5 Modeling Interactions between Quantitative and Qualitative Predictors
#

# Example: the insurance innovation example
# Table 8.4
insurance.lm2 <- lm(Y ~ X1 * X2, data = insurance.data)
summary(insurance.lm2)
anova(insurance.lm2)





############################################################################
# Chapter 9: Model Selection and Validation                                #
############################################################################


#
# 9.2 Surgical Unit Example
#

Surgical.data <- read.csv("data/CH09TA01.csv", header = F)
head(Surgical.data)
tail(Surgical.data)
colnames(Surgical.data) <- c(paste0("x", 1:8), "Y", "lnY")
dim(Surgical.data)


# Figure 9.3
Surgical.subset <- Surgical.data[1:54, c(1:4, 9, 10)]
head(Surgical.subset)
tail(Surgical.subset)
dim(Surgical.subset)
pairs(Surgical.subset)


#
# 9.3 Criteria for Model Selection
#


# Table 9.2
#install.packages("ALSM")
library(ALSM)
model.s(Surgical.subset[,1:4], Surgical.subset[, 6])


# Figure 9.4
plotmodel.s(Surgical.subset[,1:4], Surgical.subset[, 6])


#
# 9.4 Automatic Search Procedures for Model Selection
#

# Figure 9.3
# BestSub(x, y,method=c('r2','r2adj','sse','cp','press','aic','sbc'), num = 2)
# num : number of best subset model per number of predictor variable.
#model.s(Surgical.data[,1:8], Surgical.data[, 10])
plotmodel.s(Surgical.data[,1:8], Surgical.data[, 10])
BestSub(Surgical.data[,1:8], Surgical.data[, 10])



# Stepwise Regression Methods

names(Surgical.data)
Surgical.data$Y <- NULL
head(Surgical.data)


# Stepwise regression model
library(MASS)
full.model <- lm(lnY ~., data = Surgical.data)
step.model <- stepAIC(full.model, direction = "both", trace = T)
step.model
summary(step.model)

# ?stepAIC
# stepAIC(object, scope, scale = 0,
#         direction = c("both", "backward", "forward"),
#         trace = 1, keep = NULL, steps = 1000, use.start = FALSE,
#         k = 2, ...)
#




############################################################################
# Chapter 14: Logistic Regression                                          #
############################################################################


#A systems analyst studied the effect of computer programming experience on ability
#to complete within a specified time a complex programming task, including debugging.
#Twenty-five persons were selected for the study. They had varying amounts
#of programming experience (measured in months of experience)
# Y: 
# X1:
# X2: 
Task.data <- read.csv("data/CH14TA01.csv", header = F)
Task.data
colnames(Task.data) <- c("MonthExperience", "TaskSuccess", "Fitted")
str(Task.data)
Task.data$TaskSuccess <- factor(Task.data$TaskSuccess)
str(Task.data)


# fot Logistic Regression (generalized linear model)
Task.glm <- glm(TaskSuccess ~ MonthExperience,
                      data = Task.data, family = "binomial")
summary(Task.glm)



# Figure 14.5
plot(Task.data$MonthExperience, as.integer(Task.data$TaskSuccess)-1)
lines(lowess(Task.data$MonthExperience, as.integer(Task.data$TaskSuccess)-1), 
      col = 2, lwd = 2, lty = 2)

a <- min(Task.data$MonthExperience)
b <- max(Task.data$MonthExperience)
new.X <- data.frame(MonthExperience = seq(a, b, len = 50))
new.X
dim(new.X)

predicted.Y <- predict(Task.glm, new.X, type="response")
new.X <- c(new.X)$MonthExperience
lines(new.X, predicted.Y, lwd = 2, col = "black")


# odds ratio = exp(b1)
exp(coef(Task.glm))





